# coding=utf-8

import logging
from colorformatter import ColorizingStreamHandler
from .cairopainter import CairoPainter

#root = logging.getLogger()
#root.setLevel(logging.DEBUG)
#root.addHandler(ColorizingStreamHandler())
